#include <cstddef>
using std::size_t;

#include <cassert>

#include <cstring>
using std::strlen;

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <vector>
using std::vector;

#include <iostream>
using std::cout;
using std::endl;
using std::fixed;

#include <sstream>
using std::stringstream;

#include "mtrand.hpp"
mtrand R(1975);

#include "grid.hpp"

square_grid<cell> G(50);

const int delay = 5; // milliseconds

long int remaining_cells = G.length()*G.length();

class color
{
public:
	double red,green,blue;
	color() : red(0.2+0.8*R()), green(0.2+0.8*R()), blue(0.2+0.8*R()) {}
};

void glutPrint(float x, float y, const char* text, float r, float g, float b, float a) 
{ 
    if(!text || !strlen(text)) return; 
    bool blending = false; 
    if(glIsEnabled(GL_BLEND)) blending = true; 
    glEnable(GL_BLEND); 
    glColor4f(r,g,b,a); 
    glRasterPos2f(x,y); 
    while (*text) { 
        glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *text); 
        text++; 
    } 
    if(!blending) glDisable(GL_BLEND); 
}

bool is_spanning = false;
unsigned long int num_clusters = 0;

void display()
{
	static vector<color> color_list;
	const double d = 0.75/G.length();
	glClear(GL_COLOR_BUFFER_BIT);
	for (size_t i = 0; i < G.length(); ++i)
		for (size_t j = 0; j < G.length(); ++j)
			if (!G(i,j).is_removed)
			{
				const double x = 1.5*(i+0.5)/G.length() - 0.75;
				const double y = 1.5*(j+0.5)/G.length() - 0.75;
				const size_t k = G(i,j).cluster_membership-1;
				while (color_list.size() <= k)
					color_list.push_back(color());
				assert(k < color_list.size());
				glBegin(GL_POLYGON);
					glColor3f(color_list[k].red,color_list[k].blue,color_list[k].green); 
					glVertex2f(x-d,y-d);
					glVertex2f(x-d,y+d);
					glVertex2f(x+d,y+d);
					glVertex2f(x+d,y-d);
				glEnd();
				glFlush();
			}
	stringstream s;
	s.precision(3);
	s << "removal fraction = " << fixed << 1.0-double(remaining_cells)/G.length()/G.length()
	  << " clusters = " << num_clusters
	  << ( is_spanning ? " (spanning)" : "");
	glutPrint(-0.88f, -0.88f,s.str().c_str(), 1.0f, 1.0f, 1.0f, 0.5f);
	glFlush();
	glutSwapBuffers();
}

void assign_membership(size_t i, size_t j, long int m)
{
	if (!G(i,j).is_removed and G(i,j).cluster_membership == -1)
	{
		G(i,j).cluster_membership = m;
		if (i+1 < G.length())
			assign_membership(i+1,j,m);
		if (i-1 < G.length())
			assign_membership(i-1,j,m);
		if (j+1 < G.length())
			assign_membership(i,j+1,m);
		if (j-1 < G.length())
			assign_membership(i,j-1,m);
	}
}

void build_clusters(void)
{
	for (size_t i = 0; i < G.length(); ++i)
		for (size_t j = 0; j < G.length(); ++j)
			G(i,j).cluster_membership = -1;

	long int m = 0;
	for (size_t i = 0; i < G.length(); ++i)
		for (size_t j = 0; j < G.length(); ++j)
			if (!G(i,j).is_removed and G(i,j).cluster_membership == -1)
				assign_membership(i,j,++m);
	num_clusters = m;
}

void animate(int)
{
	const size_t i = R.randInt(G.length()-1);
	const size_t j = R.randInt(G.length()-1);
	if (G(i,j).is_removed == false)
	{
		G(i,j).is_removed = true;
		--remaining_cells;
	}
	
	build_clusters();
	glutPostRedisplay();
	glutTimerFunc(delay,animate,0);
}

int main(int argc, char **argv)
{
	glutInit(&argc,argv);
	glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE);
	glutInitWindowSize (640, 640);
	glutInitWindowPosition (100, 100);
	glutCreateWindow("Percolation");
	glutDisplayFunc(display);
	glutTimerFunc(delay,animate,0);
		
	build_clusters();
	
	glutMainLoop();
}
